<?php
/**
 * @subpackage  mod_xiroweb_art_nextlastlatest
 *
 * @copyright   (C) 2021-2022 Xiroweb All rights reserved.
 * @copyright   (C) 2010 Open Source Matters, Inc. <https://www.joomla.org>
 * 
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Xiroweb\Module\XirowebArtiNextlastlatest\Site\Helper;

\defined('_JEXEC') or die;

use Joomla\CMS\Access\Access;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\Component\Content\Administrator\Extension\ContentComponent;
use Joomla\Component\Content\Site\Helper\RouteHelper;
use Joomla\Utilities\ArrayHelper;

/**
 * Helper for mod_xiroweb_art_nextlastlatest
 *
 * @since  1.6
 */
abstract class XirowebArtLastnextlatestHelper
{
	/**
	 * Get a list of articles from a specific category
	 *
	 * @param   \Joomla\Registry\Registry  &$params  object holding the models parameters
	 *
	 * @return  mixed
	 *
	 * @since  1.6
	 */
	public static function getList(&$params)
	{
		$app     = Factory::getApplication();
		$factory = $app->bootComponent('com_content')->getMVCFactory();

		// Get an instance of the generic articles model
		$articles = $factory->createModel('Articles', 'Site', ['ignore_request' => true]);

		// Set application parameters in model
		$input     = $app->input;

        $option = $input->get('option');
        $view   = $input->get('view');

		$appParams = $app->getParams();
		$articles->setState('params', $appParams);
		$articles->setState('list.start', 0);
		$articles->setState('filter.published', ContentComponent::CONDITION_PUBLISHED);
		// Access filter
		$access     = !ComponentHelper::getParams('com_content')->get('show_noauth');
		$authorised = Access::getAuthorisedViewLevels(Factory::getUser()->get('id'));
		$articles->setState('filter.access', $access);

        $art_ids_exclude = array();
        
        if ($option === 'com_content')
        {
            switch ($view)
            {
              
                case 'article':

                            $article_id = $input->getInt('id');
                        
                            // Get an instance of the generic article model
                            $article = $factory->createModel('Article', 'Site', ['ignore_request' => true]);

                            $article->setState('params', $appParams);
                            $article->setState('filter.published', 1);
                            $article->setState('article.id', (int) $article_id);
                            $item   = $article->getItem();
                            $catids = array($item->catid);


                            
                            break;

                default:
                    // Return right away if not on the category or article views
                    return;
            }
        }
        else
        {
            // Return right away if not on a com_content page
            return;
        }

        if (empty($item)) 
        {
            return;
        }

        $articles->setState('filter.category_id.include', (bool) 1);
        $articles->setState('filter.category_id', $catids);
		$ordering = $params->get('date_column', 'a.created');

        switch ($ordering)
        {
          
            case 'a.created':
                $date_value = $item->created;
                break;
            case 'a.modified':
                $date_value = $item->modified;
                break;
            case 'a.publish_up':
                $date_value = $item->publish_up;
                break;

            default:
                // Return right away if not on the category or article views
                return;
        }

       

        $articles->setState('list.ordering', $ordering);
		$articles->setState('filter.featured', 'show');

        // control
        $articles->setState('filter.date_filtering', 'range');


        $articles->setState('filter.date_field', $params->get('date_column', 'a.created'));
        // Filter by language
        
        $articles->setState('filter.language', $app->getLanguageFilter());
        
        // result
        $itemsgroup = array();
        // Exclude
        
        $art_ids_exclude[] = $article_id;
        $articles->setState('filter.article_id', $art_ids_exclude);
        $articles->setState('filter.article_id.include', false);

        // latest articles
        $articles->setState('list.direction', 'DESC');
        $articles->setState('list.limit', (int) $params->get('count_latest', 4));
        $articles->setState('filter.start_date_range', $date_value);
        // $articles->setState('filter.start_date_range', $params->get('start_date_range', '1000-01-01 00:00:00'));
        $articles->setState('filter.end_date_range', $params->get('end_date_range', '9999-12-31 23:59:59'));
        $itemsgroup['latest'] = $articles->getItems();

        // next articles
        $ids = ArrayHelper::getColumn($itemsgroup['latest'], 'id');
		$ids = ArrayHelper::toInteger($ids);
        $art_ids_exclude = array_merge($ids, $art_ids_exclude);
        $articles->setState('filter.article_id', $art_ids_exclude);
        $articles->setState('list.direction', 'ASC'); 
        $articles->setState('list.limit', (int) $params->get('count_next', 4));

        $itemsgroup['next'] = $articles->getItems();
        
        // last articles
        $ids = ArrayHelper::getColumn($itemsgroup['next'], 'id');
		$ids = ArrayHelper::toInteger($ids);
        $art_ids_exclude = array_merge($ids, $art_ids_exclude);
        $articles->setState('list.direction', 'DESC');
        $articles->setState('list.limit', (int) $params->get('count_last', 4));
		$itemsgroup['last'] = $articles->getItems();

		// Display options
		$show_date        = $params->get('show_date', 0);
		$show_date_field  = $params->get('show_date_field', 'created');
		$show_date_format = $params->get('show_date_format', 'Y-m-d H:i:s');
		$show_category    = $params->get('show_category', 0);
		$show_hits        = $params->get('show_hits', 0);
		$show_author      = $params->get('show_author', 0);


        // reoder $items['next'] to DESC
        $itemsgroup['next'] = array_reverse($itemsgroup['next']);

		// Find current Article ID if on an article page
		$option = $input->get('option');
		$view   = $input->get('view');

		if ($option === 'com_content' && $view === 'article')
		{
			$active_article_id = $input->getInt('id');
		}
		else
		{
			$active_article_id = 0;
		}

		// Prepare data for display using display options
		foreach ($itemsgroup as &$items)
        {
            foreach ($items as &$item)
            {
                $item->slug = $item->id . ':' . $item->alias;

                if ($access || \in_array($item->access, $authorised))
                {
                    // We know that user has the privilege to view the article
                    $item->link = Route::_(RouteHelper::getArticleRoute($item->slug, $item->catid, $item->language));
                }
                else
                {
                    $menu      = $app->getMenu();
                    $menuitems = $menu->getItems('link', 'index.php?option=com_users&view=login');

                    if (isset($menuitems[0]))
                    {
                        $Itemid = $menuitems[0]->id;
                    }
                    elseif ($input->getInt('Itemid') > 0)
                    {
                        // Use Itemid from requesting page only if there is no existing menu
                        $Itemid = $input->getInt('Itemid');
                    }

                    $item->link = Route::_('index.php?option=com_users&view=login&Itemid=' . $Itemid);
                }

                // Used for styling the active article
                $item->active      = $item->id == $active_article_id ? 'active' : '';
                $item->displayDate = '';

                if ($show_date)
                {
                    $item->displayDate = HTMLHelper::_('date', $item->$show_date_field, $show_date_format);
                }

                if ($item->catid)
                {
                    $item->displayCategoryLink  = Route::_(RouteHelper::getCategoryRoute($item->catid, $item->category_language));
                    $item->displayCategoryTitle = $show_category ? '<a href="' . $item->displayCategoryLink . '">' . $item->category_title . '</a>' : '';
                }
                else
                {
                    $item->displayCategoryTitle = $show_category ? $item->category_title : '';
                }

                $item->displayHits       = $show_hits ? $item->hits : '';
                $item->displayAuthorName = $show_author ? $item->author : '';

            }
        }

		return $itemsgroup;
	}


    public static function parseShowValueclassConditions($showOn, $formControl = null, $group = null)
	{
		// Process the showon data.
		if (!$showOn)
		{
			return array();
		}

		$formPath = $formControl ?: '';

		if ($group)
		{
			$groups = explode('.', $group);

			// An empty formControl leads to invalid shown property
			// Use the 1st part of the group instead to avoid.
			if (empty($formPath) && isset($groups[0]))
			{
				$formPath = $groups[0];
				array_shift($groups);
			}

			foreach ($groups as $group)
			{
				$formPath .= '[' . $group . ']';
			}
		}

		$showOnData  = array();
		$showOnParts = preg_split('#(\[AND\]|\[OR\])#', $showOn, -1, PREG_SPLIT_DELIM_CAPTURE);
		$op          = '';

		foreach ($showOnParts as $showOnPart)
		{
			if (($showOnPart === '[AND]') || $showOnPart === '[OR]')
			{
				$op = trim($showOnPart, '[]');
				continue;
			}

			$compareEqual     = strpos($showOnPart, '!:') === false;
			$showOnPartBlocks = explode(($compareEqual ? ':' : '!:'), $showOnPart, 2);

			$dotPos = strpos($showOnPartBlocks[0], '.');

			if ($dotPos === false)
			{
				$field = $formPath ? $formPath . '[' . $showOnPartBlocks[0] . ']' : $showOnPartBlocks[0];
			}
			else
			{
				if ($dotPos === 0)
				{
					$fieldName = substr($showOnPartBlocks[0], 1);
					$field     = $formControl ? $formControl . '[' . $fieldName . ']' : $fieldName;
				}
				else
				{
					if ($formControl)
					{
						$field = $formControl . ('[' . str_replace('.', '][', $showOnPartBlocks[0]) . ']');
					}
					else
					{
						$groupParts = explode('.', $showOnPartBlocks[0]);
						$field      = array_shift($groupParts) . '[' . join('][', $groupParts) . ']';
					}
				}
			}

			$showOnData[] = array(
				'field'  => $field,
				'values' => explode(',', $showOnPartBlocks[1]),
				'sign'   => $compareEqual === true ? '=' : '!=',
				'op'     => $op,
			);

			if ($op !== '')
			{
				$op = '';
			}
		}

		return $showOnData;
	}

	
}
